"""Manages Snowpark Container External Volume.

Example:
    >>> new_external_volume_def = ExternalVolume(
    ...     name="MY_EXTERNAL_VOLUME",
    ...     storage_location=StorageLocationS3(
    ...         name="abcd-my-s3-us-west-2",
    ...         storage_base_url="s3://MY_EXAMPLE_BUCKET/",
    ...         storage_aws_role_arn="arn:aws:iam::123456789022:role/myrole",
    ...         encryption=Encryption(
    ...             type="AWS_SSE_KMS", kms_key_id="1234abcd-12ab-34cd-56ef-1234567890ab"
    ...         ),
    ...     ),
    ...     comment="This is my external volume",
    ... )
    >>> new_external_volume = root.external_volumes.create(new_external_volume_def)
    >>> external_volume_snapshot = new_external_volume.fetch()
    >>> external_volume_data = root.external_volumes.iter(like=”%MY_EXTERNAL_VOLUME)
    >>> new_external_volume.drop()

Refer to :class:`snowflake.core.Root` to create the ``root``.
"""

from ..external_volume._generated.models import (
    Encryption,
    ExternalVolume,
    StorageLocationAzure,
    StorageLocationGcs,
    StorageLocationS3,
    StorageLocationS3Gov,
)
from ._external_volume import ExternalVolumeCollection, ExternalVolumeResource


__all__ = [
    "ExternalVolume",
    "ExternalVolumeCollection",
    "ExternalVolumeResource",
    "StorageLocationS3",
    "StorageLocationAzure",
    "StorageLocationGcs",
    "Encryption",
    "StorageLocationS3Gov",
]



import typing

import pytest

from snowflake.core import CreateMode
from snowflake.core.user import User, UserCollection

from ..utils import random_string


if typing.TYPE_CHECKING:
    from snowflake.snowpark import Session


@pytest.mark.snowpark
@pytest.mark.use_accountadmin
def test_iter(users: UserCollection, session: "Session"):
    user_name_one = random_string(5, "test_create_user_1")
    user_name_two = random_string(5, "test_create_user_")
    user_name_three = random_string(5, "test_create_user_")

    def create_user(user_name) -> User:
        test_user = User(user_name)
        return users.create(test_user, mode=CreateMode.error_if_exists)

    try:
        test_user_one = create_user(user_name_one)
        test_user_two = create_user(user_name_two)
        test_user_three = create_user(user_name_three)

        user_names = [user.name for user in users.iter()]

        assert test_user_one.name.upper() in user_names
        assert test_user_two.name.upper() in user_names
        assert test_user_three.name.upper() in user_names

        user_names_filtered = [user.name for user in users.iter(like="test_create_user_%")]
        assert len(user_names_filtered) < len(user_names)

        user_names_filtered = [user.name for user in users.iter(like="test_create_user_1%")]
        assert test_user_one.name.upper() in user_names_filtered
        assert test_user_two.name.upper() not in user_names_filtered
        assert test_user_three.name.upper() not in user_names_filtered

        user_names_filtered = [user.name for user in users.iter(like="test_create_user_%", limit=2)]
        assert len(user_names_filtered) == 2

        user_names_filtered = [
            user.name for user in users.iter(starts_with="TEST_CREATE_USER_1", like="test_create_user_%")
        ]
        assert test_user_one.name.upper() in user_names_filtered
        assert test_user_two.name.upper() not in user_names_filtered
        assert test_user_three.name.upper() not in user_names_filtered

        users[user_name_one].drop()
        users[user_name_two].drop()
        users[user_name_three].drop()
    finally:
        for name in [user_name_one, user_name_two, user_name_three]:
            session.sql(f"DROP USER IF EXISTS {name}").collect()



"""
Cortex Inference API.

OpenAPI 3.0 specification for the Cortex REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictStr


class CompleteRequestMessagesInner(BaseModel):
    """A model object representing the CompleteRequestMessagesInner resource.

    Constructs an object of type CompleteRequestMessagesInner with the provided properties.

    Parameters
    __________
    role : str,  default 'user'
        Indicates the role of the message, one of 'system', 'user' or 'assistant'.

        Rules:
          - A 'user' message must be the last message in the list.
          - If a 'system' message is specified, it must be the first message.
          - If a 'assistant' message is specified, it must be immediately before a 'user' message in the list.

        Multiple 'assistant' and 'user' messages can be specified, but they must alternate in sequence.
    content : str, optional
        The text completion prompt, e.g. 'What is a Large Language Model?'.
    content_list : list[object], optional
        Contents of text, toolUse, toolResults, or model-specific output
    """

    role: Optional[StrictStr] = "user"

    content: Optional[StrictStr] = None

    content_list: Optional[List[Any]] = None

    __properties = ["role", "content", "content_list"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CompleteRequestMessagesInner:
        """Create an instance of CompleteRequestMessagesInner from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> CompleteRequestMessagesInner:
        """Create an instance of CompleteRequestMessagesInner from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return CompleteRequestMessagesInner.model_validate(obj)

        _obj = CompleteRequestMessagesInner.model_validate(
            {
                "role": obj.get("role") if obj.get("role") is not None else "user",
                "content": obj.get("content"),
                "content_list": obj.get("content_list"),
            }
        )

        return _obj


class CompleteRequestMessagesInnerModel:
    def __init__(
        self,  # optional properties
        role: Optional[str] = "user",
        content: Optional[str] = None,
        content_list: Optional[list[object]] = None,
    ):
        """A model object representing the CompleteRequestMessagesInner resource.

                Constructs an object of type CompleteRequestMessagesInner with the provided properties.

                Parameters
                __________
                role : str,  default 'user'
                    Indicates the role of the message, one of 'system', 'user' or 'assistant'.

        Rules:
          - A 'user' message must be the last message in the list.
          - If a 'system' message is specified, it must be the first message.
          - If a 'assistant' message is specified, it must be immediately before a 'user' message in the list.

        Multiple 'assistant' and 'user' messages can be specified, but they must alternate in sequence.

                content : str, optional
                    The text completion prompt, e.g. 'What is a Large Language Model?'.
                content_list : list[object], optional
                    Contents of text, toolUse, toolResults, or model-specific output
        """
        self.role = role
        self.content = content
        self.content_list = content_list

    __properties = ["role", "content", "content_list"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return CompleteRequestMessagesInner(
            role=self.role,
            content=self.content,
            content_list=self.content_list,
        )

    @classmethod
    def _from_model(cls, model) -> CompleteRequestMessagesInnerModel:
        return CompleteRequestMessagesInnerModel(
            role=model.role,
            content=model.content,
            content_list=model.content_list,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a CompleteRequestMessagesInner.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the CompleteRequestMessagesInner object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> CompleteRequestMessagesInnerModel:
        """Create an instance of CompleteRequestMessagesInner from a dict.

        This method constructs a CompleteRequestMessagesInner object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        CompleteRequestMessagesInner
            A CompleteRequestMessagesInner object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(CompleteRequestMessagesInner.from_dict(obj))


CompleteRequestMessagesInner._model_class = CompleteRequestMessagesInnerModel



import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.schema import Schema
from tests.utils import random_string


pytestmark = pytest.mark.usefixtures("backup_database_schema")


def test_drop(schemas):
    comment = "my comment"
    new_schema = Schema(name=random_string(5, "test_schema_"), comment=comment)
    s = schemas.create(new_schema)
    try:
        assert s.fetch().comment == comment
    finally:
        s.drop()
    s.undrop()

    try:
        assert s.fetch().comment == comment
    finally:
        s.drop()

    with pytest.raises(NotFoundError):
        s.fetch()

    # Should not error
    s.drop(if_exists=True)

    s = schemas.create(new_schema)
    try:
        assert s.fetch().comment == comment
    finally:
        s.drop()



import pytest

from snowflake.core.api_integration import ApiIntegration, GitHook
from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string


def test_drop(api_integrations):
    ai_name = random_string(10, "test_api_integration_drop_")
    ai_def = ApiIntegration(
        name=ai_name,
        api_hook=GitHook(),
        api_allowed_prefixes=["https://github.com"],
        enabled=True,
        comment="created by test_drop",
    )
    ai = api_integrations.create(ai_def)
    ai.drop()
    with pytest.raises(NotFoundError):
        ai.fetch()
    ai.drop(if_exists=True)



# coding: utf-8

"""
Preview Feature Test API

Test spec for preview feature filtering  # noqa: E501

The version of the OpenAPI document: 1.0.0
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

import json
import logging
import mimetypes
import os
import re
import tempfile
from dateutil.parser import parse
from datetime import datetime, date
from pydantic import StrictStr
from tests.preview_test._generated.models.create_resource_request import CreateResourceRequest
from tests.preview_test._generated.models.resource import Resource
from tests.preview_test._generated.models.shared_model import SharedModel
from typing import Iterable


from snowflake.core._generated.api_client import ApiClient
from tests.preview_test._generated import models

from concurrent.futures import Future
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Literal, Optional, Union, overload
from typing_extensions import Annotated

from snowflake.core._internal.utils import is_running_inside_stored_procedure
from snowflake.core._generated import rest
import snowflake.core._http_requests

from snowflake.core.exceptions import (  # noqa: F401
    _APITypeError,
    _APIValueError,
)

logger = logging.getLogger(__name__)


class DefaultApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, root, resource_class, sproc_client):
        self._root = root
        self._resource_name = "preview_test"
        self._resource_class = resource_class

        if is_running_inside_stored_procedure():
            self._api_client = sproc_client
        else:
            self._api_client = ApiClient.get_default()

    @property
    def api_client(self):
        return self._api_client

    def deserialize(self, response, response_type):
        """Deserializes response into an object.

        :param response: RESTResponse object to be deserialized.
        :param response_type: class literal for
            deserialized object, or string of class name.

        :return: deserialized object.
        """
        # handle file downloading
        # save response body into a tmp file and return the instance
        if response_type == "file":
            return self.__deserialize_file(response)

        # fetch data from response object
        try:
            data = json.loads(response.data)
        except ValueError:
            data = response.data

        return self.__deserialize(data, response_type)

    def __deserialize(self, data, klass):
        """Deserializes dict, list, str into an object.

        :param data: dict, list or str.
        :param klass: class literal, or string of class name.

        :return: object.
        """
        if data is None:
            return None

        if isinstance(klass, str):
            if klass.startswith("Iterable["):
                sub_kls = re.match(r"Iterable\[(.*)]", klass).group(1)
                return [self.__deserialize(sub_data, sub_kls) for sub_data in data]

            if klass.startswith("Dict["):
                sub_kls = re.match(r"Dict\[([^,]*), (.*)]", klass).group(2)
                return {k: self.__deserialize(v, sub_kls) for k, v in data.items()}

            # convert str to class
            if klass in snowflake.core._http_requests.NATIVE_TYPES_MAPPING:
                klass = snowflake.core._http_requests.NATIVE_TYPES_MAPPING[klass]
            else:
                klass = getattr(models, klass)

        if klass in snowflake.core._http_requests.PRIMITIVE_TYPES:
            return self.__deserialize_primitive(data, klass)
        elif klass == object:  # noqa: E721
            return self.__deserialize_object(data)
        elif klass == date:
            return self.__deserialize_date(data)
        elif klass == datetime:
            return self.__deserialize_datetime(data)
        else:
            return self.__deserialize_model(data, klass)

    def __deserialize_file(self, response):
        """Deserializes body to file

        Saves response body into a file in a temporary folder,
        using the filename from the `Content-Disposition` header if provided.

        :param response:  RESTResponse.
        :return: file path.
        """
        fd, path = tempfile.mkstemp(dir=self.api_client.configuration.temp_folder_path)
        os.close(fd)
        os.remove(path)

        content_disposition = response.getheader("Content-Disposition")
        if content_disposition:
            filename = re.search(r'filename=[\'"]?([^\'"\s]+)[\'"]?', content_disposition).group(1)
            path = os.path.join(os.path.dirname(path), filename)

        with open(path, "wb") as f:
            f.write(response.data)

        return path

    def __deserialize_primitive(self, data, klass):
        """Deserializes string to primitive type.

        :param data: str.
        :param klass: class literal.

        :return: int, long, float, str, bool.
        """
        try:
            return klass(data)
        except UnicodeEncodeError:
            return str(data)
        except TypeError:
            return data

    def __deserialize_object(self, value):
        """Return an original value.

        :return: object.
        """
        return value

    def __deserialize_date(self, string):
        """Deserializes string to date.

        :param string: str.
        :return: date.
        """
        try:
            return parse(string).date()
        except ImportError:
            return string
        except ValueError:
            raise rest.APIError(  # noqa: B904
                status=0, reason=f"Failed to parse `{string}` as date object"
            )

    def __deserialize_datetime(self, string):
        """Deserializes string to datetime.

        The string should be in iso8601 datetime format.

        :param string: str.
        :return: datetime.
        """
        try:
            return parse(string)
        except ImportError:
            return string
        except ValueError:
            raise rest.APIError(  # noqa: B904
                status=0, reason=(f"Failed to parse `{string}` as datetime object")
            )

    def __deserialize_model(self, data, klass):
        """Deserializes list or dict to model.

        :param data: dict, list.
        :param klass: class literal.
        :return: model object.
        """

        return klass.from_dict(data)

    @overload
    def create_resource(
        self, create_resource_request: CreateResourceRequest, async_req: Literal[False] = False, **kwargs
    ) -> Resource: ...

    @overload
    def create_resource(
        self, create_resource_request: CreateResourceRequest, async_req: Literal[True] = True, **kwargs
    ) -> Future[Resource]: ...

    @overload
    def create_resource(
        self, create_resource_request: CreateResourceRequest, async_req: bool = False, **kwargs
    ) -> Union[Resource, Future[Resource]]: ...

    @validate_call
    def create_resource(
        self, create_resource_request: CreateResourceRequest, **kwargs
    ) -> Union[Resource, Future[Resource]]:  # noqa: E501
        r"""Create resource (GA endpoint)  # noqa: E501


        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.create_resource(create_resource_request, async_req=True)
        >>> result = future.result()
        :param create_resource_request: (required)
        :type create_resource_request: CreateResourceRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: Union[Resource, Future[Resource]]
        """
        kwargs["_return_http_data_only"] = True
        return self.create_resource_with_http_info(create_resource_request, **kwargs)  # noqa: E501

    @validate_call
    def create_resource_with_http_info(self, create_resource_request: CreateResourceRequest, **kwargs):  # noqa: E501
        r"""Create resource (GA endpoint)  # noqa: E501


        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.create_resource_with_http_info(create_resource_request, async_req=True)
        >>> result = future.result()
        :param create_resource_request: (required)
        :type create_resource_request: CreateResourceRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: tuple(Union[Resource, Future[Resource]], status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = ["create_resource_request"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise _APITypeError(f"Got an unexpected keyword argument '{_key}' to method create_resource")
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        if _params["create_resource_request"]:
            _body_params = _params["create_resource_request"]

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get(
            "_content_type", self.api_client.select_header_content_type(["application/json"])
        )
        if _content_types_list:
            _header_params["Content-Type"] = _content_types_list

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "201": "Resource",
        }

        return self.api_client.call_api(
            self._root,
            "/api/v2/resources",
            "POST",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
            _deserialize_response_fn=self.deserialize,
        )

    @overload
    def get_resource(self, name: StrictStr, async_req: Literal[False] = False, **kwargs) -> Resource: ...

    @overload
    def get_resource(self, name: StrictStr, async_req: Literal[True] = True, **kwargs) -> Future[Resource]: ...

    @overload
    def get_resource(self, name: StrictStr, async_req: bool = False, **kwargs) -> Union[Resource, Future[Resource]]: ...

    @validate_call
    def get_resource(self, name: StrictStr, **kwargs) -> Union[Resource, Future[Resource]]:  # noqa: E501
        r"""Get resource (GA endpoint)  # noqa: E501


        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.get_resource(name, async_req=True)
        >>> result = future.result()
        :param name: (required)
        :type name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: Union[Resource, Future[Resource]]
        """
        kwargs["_return_http_data_only"] = True
        return self.get_resource_with_http_info(name, **kwargs)  # noqa: E501

    @validate_call
    def get_resource_with_http_info(self, name: StrictStr, **kwargs):  # noqa: E501
        r"""Get resource (GA endpoint)  # noqa: E501


        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.get_resource_with_http_info(name, async_req=True)
        >>> result = future.result()
        :param name: (required)
        :type name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: tuple(Union[Resource, Future[Resource]], status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = ["name"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise _APITypeError(f"Got an unexpected keyword argument '{_key}' to method get_resource")
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        if _params["name"]:
            _path_params["name"] = _params["name"]

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "200": "Resource",
        }

        return self.api_client.call_api(
            self._root,
            "/api/v2/resources/{name}",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
            _deserialize_response_fn=self.deserialize,
        )

    @overload
    def get_resource_advanced(self, name: StrictStr, async_req: Literal[False] = False, **kwargs) -> SharedModel: ...

    @overload
    def get_resource_advanced(
        self, name: StrictStr, async_req: Literal[True] = True, **kwargs
    ) -> Future[SharedModel]: ...

    @overload
    def get_resource_advanced(
        self, name: StrictStr, async_req: bool = False, **kwargs
    ) -> Union[SharedModel, Future[SharedModel]]: ...

    @validate_call
    def get_resource_advanced(self, name: StrictStr, **kwargs) -> Union[SharedModel, Future[SharedModel]]:  # noqa: E501
        r"""Get resource with advanced features (GA with controlling param)  # noqa: E501


        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.get_resource_advanced(name, async_req=True)
        >>> result = future.result()
        :param name: (required)
        :type name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: Union[SharedModel, Future[SharedModel]]
        """
        kwargs["_return_http_data_only"] = True
        return self.get_resource_advanced_with_http_info(name, **kwargs)  # noqa: E501

    @validate_call
    def get_resource_advanced_with_http_info(self, name: StrictStr, **kwargs):  # noqa: E501
        r"""Get resource with advanced features (GA with controlling param)  # noqa: E501


        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> future = api.get_resource_advanced_with_http_info(name, async_req=True)
        >>> result = future.result()
        :param name: (required)
        :type name: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _return_http_data_only: response data without head status code
                                       and headers
        :type _return_http_data_only: bool, optional
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :type _preload_content: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns a Future object representing the execution of the method.
        :rtype: tuple(Union[SharedModel, Future[SharedModel]], status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = ["name"]
        _all_params.extend(
            [
                "async_req",
                "_return_http_data_only",
                "_preload_content",
                "_request_timeout",
                "_request_auth",
                "_content_type",
                "_headers",
            ]
        )

        # validate the arguments
        for _key, _val in _params["kwargs"].items():
            if _key not in _all_params:
                raise _APITypeError(f"Got an unexpected keyword argument '{_key}' to method get_resource_advanced")
            _params[_key] = _val
        del _params["kwargs"]

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        if _params["name"]:
            _path_params["name"] = _params["name"]

        # process the query parameters
        _query_params = []

        # process the header parameters
        _header_params = dict(_params.get("_headers", {}))

        # process the form parameters
        _form_params = []
        _files = {}

        # process the body parameter
        _body_params = None

        # set the HTTP header `Accept`
        _header_params["Accept"] = self.api_client.select_header_accept(["application/json"])  # noqa: E501

        # authentication setting
        _auth_settings = []  # noqa: E501

        _response_types_map = {
            "200": "SharedModel",
        }

        return self.api_client.call_api(
            self._root,
            "/api/v2/resources/{name}/advanced",
            "GET",
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get("async_req"),
            _return_http_data_only=_params.get("_return_http_data_only"),  # noqa: E501
            _preload_content=_params.get("_preload_content", True),
            _request_timeout=_params.get("_request_timeout"),
            collection_formats=_collection_formats,
            _request_auth=_params.get("_request_auth"),
            _deserialize_response_fn=self.deserialize,
        )



from tests.integ.table.conftest import assert_table


def test_fetch(tables, table_handle, database, schema):
    table_deep = table_handle.fetch()
    assert_table(table_deep, table_handle.name, database, schema, True)



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.artifact_repository._generated.api.artifact_repository_api import ArtifactRepositoryApi

__all__ = [
    "ArtifactRepositoryApi",
]



from contextlib import suppress

import pytest as pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.notebook import Notebook
from tests.utils import random_string


@pytest.mark.min_sf_ver("8.37.0")
def test_execute_notebook(cursor, executable_notebook, tables, table_name_for_execute_notebook_test):
    test_table_handle = tables[table_name_for_execute_notebook_test]

    try:
        assert len(list(tables.iter(like=table_name_for_execute_notebook_test))) == 0

        # Now, execute the notebook; it should create the test table
        executable_notebook.execute()

        # Now, the table should be there
        assert len(list(tables.iter(like=table_name_for_execute_notebook_test))) == 1
    finally:
        with suppress(NotFoundError):
            test_table_handle.drop()


@pytest.mark.min_sf_ver("8.37.0")
def test_live_versions_and_commit_notebook(notebooks):
    notebook_name = random_string(5, "test_notebook_stage_")
    notebook = Notebook(name=notebook_name)
    notebook_handle = notebooks[notebook.name]

    notebooks.create(notebook)

    try:
        # Initially, there should be no live version
        assert notebook_handle.fetch().live_version_location_uri is None

        # This should add a LIVE version
        notebook_handle.add_live_version(from_last=True)
        assert notebook_handle.fetch().live_version_location_uri is not None

        # Committing sets the LIVE version to null
        notebook_handle.commit()
        assert notebook_handle.fetch().live_version_location_uri is None

    finally:
        with suppress(NotFoundError):
            notebook_handle.drop()



import pytest

from snowflake.core.role import ContainingScope, Securable
from snowflake.core.schema import Schema
from snowflake.core.table import Table, TableColumn
from tests.utils import random_string


pytestmark = pytest.mark.min_sf_ver("8.39.0")


@pytest.mark.use_accountadmin
def test_grant_role(roles, test_role_name, test_role_name_2, test_database_role_name, test_database_for_grant_name):
    roles[test_role_name].grant_role(role_type="ROLE", role=Securable(name=test_role_name_2))

    roles[test_role_name].grant_role(
        role_type="DATABASE ROLE", role=Securable(name=test_database_role_name, database=test_database_for_grant_name)
    )

    # running the same grant again should not be an issue
    roles[test_role_name].grant_role(role_type="ROLE", role=Securable(name=test_role_name_2))

    grants = list(roles[test_role_name].iter_grants_to())
    for grant in grants:
        assert grant.securable_type in ["ROLE", "DATABASE ROLE"]
    assert len(grants) == 2


@pytest.mark.use_accountadmin
def test_grant_privileges(
    roles, test_role_name, test_database_for_grant_name, test_schema_for_grant_name, test_table_for_grant_name
):
    # without securable name (to this account)
    roles[test_role_name].grant_privileges(
        privileges=["CREATE DATABASE", "CREATE ACCOUNT", "CREATE WAREHOUSE"], securable_type="ACCOUNT"
    )

    # grant on a specific database
    roles[test_role_name].grant_privileges(
        privileges=["MODIFY", "MONITOR"],
        securable_type="DATABASE",
        securable=Securable(name=test_database_for_grant_name),
    )

    # grant on a schema
    roles[test_role_name].grant_privileges(
        privileges=["CREATE TASK"],
        securable_type="SCHEMA",
        securable=Securable(database=test_database_for_grant_name, name=test_schema_for_grant_name),
    )

    # grant on a table
    roles[test_role_name].grant_privileges(
        privileges=["SELECT"],
        securable_type="TABLE",
        securable=Securable(
            database=test_database_for_grant_name, schema=test_schema_for_grant_name, name=test_table_for_grant_name
        ),
    )

    assert len(list(roles[test_role_name].iter_grants_to())) == 7


@pytest.mark.use_accountadmin
def test_grant_privileges_with_grant_option(
    roles, test_role_name, test_database_for_grant_name, test_schema_for_grant_name, test_table_for_grant_name
):
    # without securable name (to this account)
    roles[test_role_name].grant_privileges(
        privileges=["CREATE DATABASE", "CREATE ACCOUNT", "CREATE WAREHOUSE"],
        securable_type="ACCOUNT",
        grant_option=True,
    )

    # grant on a specific database
    roles[test_role_name].grant_privileges(
        privileges=["MODIFY", "MONITOR"],
        securable_type="DATABASE",
        securable=Securable(name=test_database_for_grant_name),
        grant_option=True,
    )

    # grant on a schema
    roles[test_role_name].grant_privileges(
        privileges=["CREATE TASK"],
        securable_type="SCHEMA",
        securable=Securable(database=test_database_for_grant_name, name=test_schema_for_grant_name),
        grant_option=True,
    )

    # grant on a table
    roles[test_role_name].grant_privileges(
        privileges=["SELECT"],
        securable_type="TABLE",
        securable=Securable(
            database=test_database_for_grant_name, schema=test_schema_for_grant_name, name=test_table_for_grant_name
        ),
        grant_option=True,
    )

    grant_list = list(roles[test_role_name].iter_grants_to())
    assert len(grant_list) == 7

    for grant in grant_list:
        assert grant.grant_option


@pytest.mark.use_accountadmin
def test_grant_privileges_on_all(roles, test_role_name, test_database_for_grant_name, test_schema_for_grant_name):
    # all schemas in a database
    roles[test_role_name].grant_privileges_on_all(
        privileges=["USAGE"],
        securable_type="SCHEMA",
        containing_scope=ContainingScope(database=test_database_for_grant_name),
    )

    # all tables in a schema
    roles[test_role_name].grant_privileges_on_all(
        privileges=["SELECT"],
        securable_type="TABLE",
        containing_scope=ContainingScope(database=test_database_for_grant_name, schema=test_schema_for_grant_name),
    )

    grants = list(roles[test_role_name].iter_grants_to())
    for grant in grants:
        if grant.securable_type == "SCHEMA":
            assert grant.securable.database == test_database_for_grant_name.upper()
            assert grant.securable.name is not None
        else:
            assert grant.securable_type == "TABLE"
            assert grant.securable.database == test_database_for_grant_name.upper()
            assert grant.securable.var_schema == test_schema_for_grant_name.upper()
            assert grant.securable.name is not None
    assert len(grants) >= 2


@pytest.mark.use_accountadmin
def test_grant_future_privileges(
    roles,
    test_role_name,
    test_database_for_grant_name,
    test_schema_for_grant_name,
    backup_database_schema,
    schemas_in_test_database,
    tables_in_test_database,
):
    del backup_database_schema
    # all future schemas in a database
    roles[test_role_name].grant_future_privileges(
        privileges=["USAGE"],
        securable_type="SCHEMA",
        containing_scope=ContainingScope(database=test_database_for_grant_name),
    )

    # all future tables in a schema
    roles[test_role_name].grant_future_privileges(
        privileges=["INSERT"],
        securable_type="TABLE",
        containing_scope=ContainingScope(database=test_database_for_grant_name, schema=test_schema_for_grant_name),
    )

    assert len(list(roles[test_role_name].iter_grants_to())) == 0

    # create a new schema and table
    try:
        schema_name = random_string(4, "test_grant_future_privileges_schema_")
        schemas_in_test_database.create(Schema(name=schema_name))
        table_name = random_string(5, "test_grant_future_privileges_table_")
        columns = [TableColumn(name="col1", datatype="int"), TableColumn(name="col2", datatype="string")]
        test_table = Table(name=table_name, columns=columns)
        tables_in_test_database.create(test_table)

        assert len(list(roles[test_role_name].iter_grants_to())) == 2
    finally:
        schemas_in_test_database[schema_name].drop()
        tables_in_test_database[table_name].drop()



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.managed_account._generated.api.managed_account_api import ManagedAccountApi

__all__ = [
    "ManagedAccountApi",
]



# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.

import time

from contextlib import suppress

import pytest

from snowflake.core.exceptions import APIError, NotFoundError
from snowflake.core.warehouse import Warehouse

from ..utils import random_string


pytestmark = pytest.mark.usefixtures("backup_warehouse_fixture")


def test_use_warehouse(warehouses, backup_warehouse_fixture, cursor):
    warehouse_name = random_string(5, "test_use_warehouse_")
    test_warehouse = Warehouse(name=warehouse_name)
    warehouse_ref = None

    try:
        warehouse_ref = warehouses.create(test_warehouse)

        # Creating a warehouse may set the active warehouse to the new warehouse; set it back to the original,
        # if one was already active.
        if backup_warehouse_fixture is not None:
            warehouses[backup_warehouse_fixture].use_warehouse()
            cur_warehouse = cursor.execute("SELECT current_warehouse()").fetchone()[0]
            assert cur_warehouse.upper() == backup_warehouse_fixture.upper()

        # Test use warehouse on the new warehouse
        warehouse_ref.use_warehouse()

        cur_warehouse = cursor.execute("SELECT current_warehouse()").fetchone()[0]
        assert cur_warehouse.upper() == warehouse_name.upper()

    finally:
        with suppress(NotFoundError):
            warehouse_ref.drop()


def test_abort_all_queries(warehouses, session):
    warehouse_name = random_string(5, "test_abort_all_queries_warehouse_")
    test_warehouse = Warehouse(name=warehouse_name)

    warehouse_ref = None
    try:
        warehouse_ref = warehouses.create(test_warehouse)
        result = next(warehouses.iter(like=warehouse_name))
        time.sleep(5)
        warehouse_ref.abort_all_queries()
        time.sleep(5)
        result = next(warehouses.iter(like=warehouse_name))
        assert result.running == 0 and result.queued == 0

        warehouse_ref.drop()
        warehouse_ref.abort_all_queries(if_exists=True)

    finally:
        with suppress(NotFoundError):
            warehouse_ref.drop()


def test_suspend_and_resume(warehouses):
    warehouse_name = random_string(5, "test_suspend_and_resume_warehouse_")
    test_warehouse = Warehouse(name=warehouse_name)

    warehouse_ref = None
    try:
        warehouse_ref = warehouses.create(test_warehouse)
        # Test warehouse suspend from default state
        warehouse_ref.suspend()
        result = next(warehouses.iter(like=warehouse_name))
        assert result.state in ("SUSPENDED", "SUSPENDING")

        # Test warehouse resume from suspended state
        warehouse_ref.resume()
        result = next(warehouses.iter(like=warehouse_name))
        assert result.state in ("STARTING", "STARTED", "RESUMING")

        # suspend again from resumed state
        warehouse_ref.suspend()
        result = next(warehouses.iter(like=warehouse_name))
        assert result.state in ("SUSPENDED", "SUSPENDING")

        # suspend when it is already suspended
        with pytest.raises(APIError):
            warehouse_ref.suspend()

        warehouse_ref.drop()

        # These should not error
        warehouse_ref.suspend(if_exists=True)
        warehouse_ref.resume(if_exists=True)

        warehouse_ref = warehouses.create(test_warehouse)
        result = next(warehouses.iter(like=warehouse_name))
        assert result.state in ("STARTING", "STARTED", "RESUMING")

        # Warehouse can be resumed even if running because IF SUSPENDED is used in REST
        warehouse_ref.resume()

    finally:
        with suppress(NotFoundError):
            warehouse_ref.drop()
